package utility;
import java.io.File;
import java.io.IOException;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;

import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttCallback;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import scenari.Automa;
import scenari.Scenari;

public class SubscribeCallback implements MqttCallback{
	
	private Scenari scenari;
//	private Publisher publisher;
	private String learnTrigger;
	private String attivaScenari;
	private Esecutore esec;
	private Automa automa;
	
	public SubscribeCallback(Scenari scenari/*, Publisher publisher*/, String learnTrigger, String attivaScenari, Esecutore esec, Automa automa) {
		this.scenari = scenari;
//		this.publisher = publisher;
		this.learnTrigger = learnTrigger;
		this.attivaScenari = attivaScenari;
		this.esec = esec;
		this.automa = automa;
	}

	@Override
	public void connectionLost(Throwable arg0) {
		System.out.println("connessione persa\t"+arg0);
//		boolean retry = true;
		final Date d = new Date();
		Date d2 = new Date();
		long time = Math.abs(d2.getTime()-d.getTime());
		
		while(/*retry ||*/ (time<600000)) {
			try {
				System.out.println("retrying...");
				
				Scenari.main(null);
				return;
//				esec.interrupt();
//				publisher.interrupt();
//				publisher = new Publisher(new MyQueue<Pair>(), scenari);
//				esec = new Esecutore(publisher,automa);
				
//				scenari.startClient(esec, publisher);
//				retry = false;
			} catch (MqttException | JSONException | IOException e) {
				System.out.println("non sono riusciuto");
				d2 = new Date();
				time = Math.abs(d2.getTime()-d.getTime());
			}
		}
		System.out.println("fuori dal while");
		if(time>=600000) {
			System.out.println("Tentativo di riconnessione fallito");
			System.exit(1);
		}
	}

	
	@Override
	public void deliveryComplete(IMqttDeliveryToken arg0) {
		// Nessuna operazione
		System.out.println("inviato");
	}
	

	@Override
	public void messageArrived(String topic, MqttMessage message) throws MqttException, JSONException, IOException {
//		topic = topic.toLowerCase();
		System.out.println("new message: "+topic+"  "+message.toString());
		if(topic.equals("to/all")) {
			System.out.println("invio lo stato dell'automa");
			sendRequested("from/"+Scenari.getMqttTree("scenari/description"),false);
			return;
		}
		if(topic.equals("rpc/"+Scenari.getMqttTree("scenari"))) {
			System.out.println("invio la mia configurazione");
			sendRequested("from/"+Scenari.getMqttTree("scenari"),true); // CONTROLLA SE VA BENE PUBBLICARE LO STATO SU QUESTO TOPIC O SE IL TOPIC DEVE ESSERE UN'ALTRO. NOTA: NON POSSO PUBBLICARE SULLO STESSO TOPIC CHE HO SOTTOSCRITTO
			return;
		}
		
		
		JSONObject msgJson = new JSONObject(message.toString());
		// CONTROLLA SULLA BEAGLEBONE CHE IL MESSAGGIO MANDATO SIA UN INT. CONTROLLA ANCHE IL TOPIC SU CUI MANDA I MESSAGGI (da aver sottoscritto i topic correttamente)
		
//		gli event sono 0 o 1; i trigger quando sono attivati/premuti inviano uno 0, che e' l'unico elemento che mi interessa
		if(topic.equals("from/"+Scenari.getMqttTree("gpio/"+learnTrigger))) {
			System.out.println("trigger per learn\t"+!automa.getAntifurto());
			if(automa.getAntifurto())//se l'antifurto e' attivo, non posso attivare gli scenari
				return;
			int newStatus = msgJson.getInt("event");
			if(newStatus != 1)
				return;
			handleEvent(newStatus,learnTrigger);
			return;
		}
		if(topic.equals("from/"+Scenari.getMqttTree("gpio/"+attivaScenari))) {
			System.out.println("trigger per antifurto\t"+!automa.getLearn());
			if(automa.getLearn())//se l'apprendimento e' attivo, non posso attivare l'antifurto
				return;
			int newStatus = msgJson.getInt("event");
			if(newStatus != 1)
				return;
			handleEvent(newStatus,attivaScenari);
			return;
		}
		if(topic.startsWith("from/"+Scenari.getMqttTree("gpio/IN"))) {
			System.out.println("totalmente inutile per me!");
			return;//non mi interessa altro da IN
		}
//		if(topic.equals("to/"+Scenari.getMqttTree()+"scenari/learn")) {//mi aspetto un {learn:true/false}, scritto da handleEvent
//			System.out.println("messaggio per iniziare/terminare a imparare");
//			boolean val = msgJson.getBoolean("learn");
//			handleLearn(val);
//			return;
//		}
		if(topic.startsWith("from/"+Scenari.getMqttTree("gpio/OUT"))) {
			if(message.toString().contains("event")) {
				System.out.println("se learn==true, salvo questo event!");
				System.out.println(automa.getLearn()+"\t"+msgJson.toString());
			}
			if(!automa.getLearn() || !message.toString().contains("event"))
				return;
			if(scenarioName == null) {
				handleLearn(true);
			}
			int val = msgJson.getInt("event");
			String luce = topic.substring(23);
			handleLearnData(luce,val);
			return;
		}
		if(topic.startsWith("to/"+Scenari.getMqttTree("scenari"))) {
			System.out.println("Cambio quale scenario e' attivo, se antifurto attivo");
			if(!automa.getAntifurto() || !message.toString().contains("event") || !message.toString().contains("nome"))
				return;
			changeActiveScenario(new JSONObject(message.toString()));
			return;
		}
		
		System.out.println("Impossibile");
	}
	
	private void handleEvent(int newStatus,String which) throws JSONException, MqttException{
//		if(newStatus != 1)
//			return;
		String cosa = which == learnTrigger ? "learn" : "antifurto";
		String cmd = automa.findPossibleCommand(cosa);
		if(cmd == null)
			return;
		System.out.println("handling event: "+cosa+"\t"+cmd);
		automa.setStato(new JSONObject().put(cosa,cmd).toString());
		System.out.println("stato\t"+automa.getStato());
		if(Helper.compareText(cosa, "learn")) {
//			JSONObject j = new JSONObject().put("learn", Helper.compareText(cmd, "on"));
//			Helper.sendMqttMessage(scenari.mqttClient, "to/"+Scenari.getMqttTree()+"scenari/learn", j.toString());
//			publisher.aggiungiComando("to/"+Scenari.getMqttTree()+"scenari/learn", j.toString());//NON posso inviare ad un topic che subbo
			boolean val = Helper.compareText(cmd, "on");
			handleLearn(val);
		}
//		if(newStatus!=-1) {
//		String statusOnOff = automa.converter(newStatus); // lo stato e' della forma "on" oppure "off"
//		if(! statusOnOff.equals(automa.getStatoSuono()) ) {
//			automa.aggiornaStatoSuono(statusOnOff);
//		}
	}
	
	private String scenarioName = null;//"scenario"+LocalTime.now();
//	private LocalTime lastTime = null;
	
	private void handleLearn(boolean shouldLearn) {
		System.out.println("handleLearn "+shouldLearn+" ; "+automa.getLearn());
		automa.setStato("{\"learn\":"+ (shouldLearn ? "\"on\"" : "\"off\"")+"}");
		System.out.println("\t"+automa.getLearn());
		if(automa.getLearn()) {
			if(/*lastTime == null &&*/ scenarioName == null) {
//				lastTime = LocalTime.now();
				scenarioName = "scenario-"+LocalDateTime.now().toString().replace(':', '-')+".json";
			}
		} else {
//			lastTime = null;
			scenarioName = null;
//			TODO tell antifurto to turn off if shouldLearn == true
		}
	}
	
	private void handleLearnData(String luce, int val) throws MqttException, IOException, JSONException {
		JSONObject j = new JSONObject();
//		tempo passato dallo scorso comando
		LocalTime now = LocalTime.now();
//		long diff = Helper.timeDifference(lastTime,now);
		j.put("tempo", now.format(DateTimeFormatter.ofPattern("HH-mm-ss")));
//		lastTime = now;
//		quale luce
		j.put("luce", luce);
//		accesa o spenta?
		j.put("cmd", val);
		Helper.appendiFile(j, "./db-scenari/"+scenarioName,false);
	}
	
	private void changeActiveScenario(JSONObject scenario) throws IOException, JSONException, MqttException {
		String nuovoScenario = esec.getUCS().requestToChange(scenario);
		System.out.println(nuovoScenario);
		JSONObject j = new JSONObject();
		j.put("nome", nuovoScenario);
		scenari.sendMqttMessage("from/"+Scenari.getMqttTree("/scenari/attiva"), j.toString());
	}
	
	private void sendRequested(String topic,boolean moreInfo) throws MqttException, JSONException, IOException {
		JSONObject j = new JSONObject();
		j.put("stato", automa.getStato());
		if(moreInfo) {
			j.put("learn", automa.getLearn());
			boolean antifurto = automa.getAntifurto();
			j.put("antifurto", antifurto);
			j.put("learn-trigger", learnTrigger);
			j.put("attiva-scenari", attivaScenari);
//			invio tutti gli scenari
			{
				ArrayList<File> allScenariFile = esec.getAllScenariAvailable();
				JSONArray arr = new JSONArray();
				for(File f : allScenariFile) {
					JSONObject scenarioData = new JSONObject();
					System.out.println(esec.getScenarioNome(f));
					scenarioData.put("nome", esec.getScenarioNome(f).substring(0,28));
					scenarioData.put("data", esec.getScenarioData(f));
					arr.put(scenarioData);
				}
				j.put("scenari-disponibili", arr);
			}
//			invio lo scenario attivo
			if(antifurto) {
				String scenarioAttivo = esec.getScenarioAttivo();
				if(scenarioAttivo != null && !scenarioAttivo.equals("none")) {
					JSONObject scenarioValori = new JSONObject();
					System.out.println(scenarioAttivo);
					scenarioValori.put("nome", scenarioAttivo.substring(0,28));
					scenarioValori.put("data", esec.getDataScenarioAttivo());
					j.put("scenario-attivo", scenarioValori);
				}
			}
		}
//		Helper.sendMqttMessage(scenari.mqttClient, topic, j.toString());
//		publisher.aggiungiComando(topic,j.toString());
		scenari.sendMqttMessage(topic, j.toString());
	}
	
//	private void sendDescription(String topic) throws JSONException, MqttException {
		// DA FARE: guardare quale descrizione manda netmon, archiver, meter,... e implementare di conseguenza questo metodo
//		String msg = 
//		publisher.aggiungiComando(topic, msg); 
//	}
	

}
